//A Dynamic Model of a ThermoStat By Wyatt Houtz
#include "iostream.h"
#include "fstream.h"
#include "iomanip.h"


class Chaos {
	private:
   	typedef long double decimal;
      decimal *TIME, *TR, *TE, *THETA, *OMEGA, StepSize;
      decimal Itheta, Itr, Ite, Iomega;
		decimal LeftSlope, LeftStart, LeftEnd;
      decimal RightSlope, RightStart, RightEnd;
      double He, Hr, q, C, Tau;
   	long int N, el, Total, SlowOmega;
      bool file, buffer;
      char Z[500];

   public:
   	Chaos();	  //Default Constructor
      ~Chaos();  //Default Destructor
      void EnterData();
      void Run();
      void DataFile();
      void DataBuffer();
      decimal Function_Left(decimal x); //left Function
      decimal Function_Right(decimal x); //Right Function

   };

Chaos::Chaos() {   //Constructor Initializes program

	EnterData();

	el = (long int) Tau / StepSize + 1;
   Total = N + el;

   //Arrays
   TIME = new decimal[Total];
  	THETA = new decimal[Total];
   TR = new decimal[Total];
  	TE = new decimal[Total];
   OMEGA = new decimal[Total];


   //Initializes arrays with specified values.
   for (long int k=0; k < el; k++) {
      *(THETA + k) = Itheta;
      *(TR + k) = Itr;
      *(TE + k) = Ite;
      *(OMEGA + k) = Iomega;
      }
	for (long int k=0; k < el + N; k++)
      *(TIME + k) = k * StepSize;

	}

Chaos::~Chaos() {
   delete [ ] TIME;
   delete [ ] THETA;
   delete [ ] TR;
   delete [ ] TE;
   delete [ ] OMEGA;

	}


long double Chaos::Function_Left(decimal x) {
   LeftEnd = (decimal) 1/LeftSlope + LeftStart;
   if (x <= LeftStart)
   	return 0;
   else if (x >= LeftEnd)
   	return 1;
   else
   	return LeftSlope * (x - LeftStart);
	}

long double Chaos::Function_Right(decimal x) {
   RightEnd = (decimal) 1/RightSlope + RightStart;
   if (x <= RightStart)
   	return 0;
   else if (x >= RightEnd)
   	return 1;
   else
   	return RightSlope * (x - RightStart);
	}

void Chaos::EnterData() {

	ifstream infile("Initials.txt", ios::in);

   infile >> q >> Z >> He >> Z >> Hr >> Z >> Tau >> Z >> C >> Z
          >> SlowOmega >> Z >> Itheta >> Z >> Ite >> Z >> Itr >> Z
          >> Iomega >> Z >> LeftStart >> Z >> LeftSlope >> Z
          >> RightStart >> Z >> RightSlope >> Z >> N >> Z >> StepSize >> Z
          >> file >> Z >> buffer;
	if (buffer) {
   	cout   << "q = " << q << endl
   			 << "He = " << He << endl
         	 << "Hr = " << Hr << endl
	          << "Tau = " << Tau << endl
   	       << "C = " << C << endl
      	    << "\nSlowOmega = " << SlowOmega << endl
         	 << "Initial Theta = " << Itheta << endl
	          << "Initial Te = " << Ite << endl
   	       << "Initial Tr = " << Itr << endl
      	    << "Initial Omega = " << Iomega << endl
         	 << "\nLeftStart = " << LeftStart << endl
	          << "LeftSlope = " << LeftSlope << endl
   	       << "RightStart = " << RightStart << endl
      	    << "RightSlope = " << RightSlope << endl
         	 << "\nN = " << N << endl
	          << "StepSize = " <<StepSize << endl;
   	}
   infile.close();
	}

void Chaos::Run() {
	for (long int i = el; i < Total; i++) {
		*(THETA + i) = (1 - He*StepSize) * *(THETA + i - 1)
      					  + (He*StepSize) * *(TE + i - 1);

      if (*(THETA + i) == *(THETA + i - 1))
      	*(OMEGA + i) = *(OMEGA + i - 1);

      else if (*(THETA + i) > *(THETA + i - 1)) {
      	if (Function_Right(*(THETA + i)) >= *(OMEGA + i - 1))
         	*(OMEGA + i) = Function_Right(*(THETA + i));
         else
         	*(OMEGA + i) = *(OMEGA + i - 1);
         }

      else if (*(THETA + i) < *(THETA + i - 1)) {
      	if (Function_Left(*(THETA + i)) <= *(OMEGA + i - 1))
         	*(OMEGA + i) = Function_Left(*(THETA + i));
         else
         	*(OMEGA + i) = *(OMEGA + i - 1);
         }

/*    if (*(OMEGA + i) > 1)
      	*(OMEGA + i) = 1;

      if (*(OMEGA + i) < 0)
      	*(OMEGA + i) = 0;
*/

      *(TE + i) = (1 - C*StepSize)* *(TE + i - 1)
      				+ C * StepSize * *(OMEGA + i - SlowOmega) *
                  (*(TR + i - el) - *(TE + i - el))
                  + C * StepSize * *(TE + i - el)
                  + StepSize*q;

      *(TR + i) = (1 - (*(OMEGA + i - SlowOmega) + Hr)* StepSize) *
      				*(TR + i - 1) + *(OMEGA + i - SlowOmega) * StepSize
                  * *(TE + i - el);
   	}
	}

void Chaos::DataFile() {
	if (file) {
	   ofstream outfile("Results.doc");
   	for (long int i = el; i < Total; i++) {
      	outfile << setiosflags(ios::fixed) << setiosflags(ios::showpoint)
      			  << setprecision(5) << *(THETA + i) << ", "
      			  << setprecision(5) << *(TE + i) << ", "
	              << setprecision(5) << *(TR + i) << ", "
   	           << setprecision(5) << *(OMEGA + i) << ", "
      	        << setprecision(5) << *(TIME + i) << endl;
	   	}
   	outfile.close();
		}
   }

void Chaos::DataBuffer() {
	if (buffer) {
	   for (long int i = el; i < Total; i++) {
   	   cout << setiosflags(ios::showpoint) //<< setiosflags(ios::fixed)
      			  << setprecision(5) << *(THETA + i) << ", "
      			  << setprecision(5) << *(TE + i) << ", "
            	  << setprecision(5) << *(TR + i) << ", "
	              << setprecision(5) << *(OMEGA + i) << ", "
   	           << setprecision(5) << *(TIME + i) << endl;
	   	}
		}
   }

void main() {
   Chaos obj;
   obj.Run();
   obj.DataFile();
   obj.DataBuffer();
	}







